@@ -48,6 +48,10 @@ struct ngx_cycle_s {
     ngx_connection_t         *free_connections;
     ngx_uint_t                free_connection_n;
 
+    ngx_module_t            **modules;
+    ngx_uint_t                modules_n;
+    ngx_uint_t                modules_used;    /* unsigned  modules_used:1; */
+
     ngx_queue_t               reusable_connections_queue;
 
     ngx_array_t               listening;
@@ -75,35 +79,35 @@ struct ngx_cycle_s {
 
 
 typedef struct {
-     ngx_flag_t               daemon;
-     ngx_flag_t               master;
+    ngx_flag_t                daemon;
+    ngx_flag_t                master;
 
-     ngx_msec_t               timer_resolution;
+    ngx_msec_t                timer_resolution;
 
-     ngx_int_t                worker_processes;
-     ngx_int_t                debug_points;
+    ngx_int_t                 worker_processes;
+    ngx_int_t                 debug_points;
 
-     ngx_int_t                rlimit_nofile;
-     off_t                    rlimit_core;
+    ngx_int_t                 rlimit_nofile;
+    off_t                     rlimit_core;
 
-     int                      priority;
+    int                       priority;
 
-     ngx_uint_t               cpu_affinity_auto;
-     ngx_uint_t               cpu_affinity_n;
-     uint64_t                *cpu_affinity;
+    ngx_uint_t                cpu_affinity_auto;
+    ngx_uint_t                cpu_affinity_n;
+    ngx_cpuset_t             *cpu_affinity;
 
-     char                    *username;
-     ngx_uid_t                user;
-     ngx_gid_t                group;
+    char                     *username;
+    ngx_uid_t                 user;
+    ngx_gid_t                 group;
 
-     ngx_str_t                working_directory;
-     ngx_str_t                lock_file;
+    ngx_str_t                 working_directory;
+    ngx_str_t                 lock_file;
 
-     ngx_str_t                pid;
-     ngx_str_t                oldpid;
+    ngx_str_t                 pid;
+    ngx_str_t                 oldpid;
 
-     ngx_array_t              env;
-     char                   **environment;
+    ngx_array_t               env;
+    char                    **environment;
 } ngx_core_conf_t;
 
 
@@ -117,7 +121,7 @@ ngx_int_t ngx_signal_process(ngx_cycle_t *cycle, char *sig);
 void ngx_reopen_files(ngx_cycle_t *cycle, ngx_uid_t user);
 char **ngx_set_environment(ngx_cycle_t *cycle, ngx_uint_t *last);
 ngx_pid_t ngx_exec_new_binary(ngx_cycle_t *cycle, char *const *argv);
-uint64_t ngx_get_cpu_affinity(ngx_uint_t n);
+ngx_cpuset_t *ngx_get_cpu_affinity(ngx_uint_t n);
 ngx_shm_zone_t *ngx_shared_memory_add(ngx_conf_t *cf, ngx_str_t *name,
     size_t size, void *tag);
 
