@@ -204,7 +204,7 @@ ngx_http_static_handler(ngx_http_request_t *r)
 
 #endif
 
-    if (r->method & NGX_HTTP_POST) {
+    if (r->method == NGX_HTTP_POST) {
         return NGX_HTTP_NOT_ALLOWED;
     }
 
