@@ -32,6 +32,7 @@ static ngx_os_io_t ngx_freebsd_io = {
     ngx_readv_chain,
     ngx_udp_unix_recv,
     ngx_unix_send,
+    ngx_udp_unix_send,
 #if (NGX_HAVE_SENDFILE)
     ngx_freebsd_sendfile_chain,
     NGX_IO_SENDFILE
@@ -72,13 +73,13 @@ sysctl_t sysctls[] = {
 
 
 void
-ngx_debug_init()
+ngx_debug_init(void)
 {
 #if (NGX_DEBUG_MALLOC)
 
-#if __FreeBSD_version >= 500014
+#if __FreeBSD_version >= 500014 && __FreeBSD_version < 1000011
     _malloc_options = "J";
-#else
+#elif __FreeBSD_version < 500014
     malloc_options = "J";
 #endif
 
